/*
 * MainView.java
 *
 * Created on 2008/05/29, 18:01
 */
package p4uclient;

import java.awt.event.ActionEvent;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.SwingWorker;
import javax.swing.UIManager;

/**
 * アプリのインターフェイスを管理
 * @author  tondol
 */
public class MainView extends javax.swing.JFrame {

    private static MainView instance = null;
    FileTableModel modelSearch = null;
    TaskTableModel modelTasks = null;
    // UIの状態を保持
    private int stateBusy = 0;
    private boolean stateOnline = false;
    private boolean stateScan = false;

    /** Creates new form MainView */
    private MainView() {
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
            SwingUtilities.updateComponentTreeUI(this);
        } catch (Exception e) {
        } finally {
            modelSearch = new FileTableModel();
            modelTasks = new TaskTableModel();
            initComponents();
            applyState();
        }
    }

    /**
     * インスタンスを生成しつつ表示
     */
    public static MainView getInstance() {
        if (instance == null) {
            instance = new MainView();
            java.awt.EventQueue.invokeLater(new Runnable() {

                public void run() {
                    instance.setVisible(true);
                }
            });
        }
        return instance;
    }

    /**
     * フラグを変更
     */
    public void setBusy(boolean b) {
        if (b) {
            stateBusy++;
        } else {
            stateBusy--;
        }
    }

    public void setOnline(boolean b) {
        stateOnline = b;
    }

    public void setScan(boolean b) {
        stateScan = b;
    }

    /**
     * フラグ状態をUIに適用
     */
    public void applyState() {
        java.awt.EventQueue.invokeLater(new Runnable() {

            public void run() {
                if (stateBusy > 0) {
                    connectMenuItem.setEnabled(false);
                    disconnectMenuItem.setEnabled(false);
                    scanMenuItem.setEnabled(false);
                    txtSearch.setEnabled(false);
                    btnSearch.setEnabled(false);
                    btnDownload.setEnabled(false);
                    btnCancel.setEnabled(false);
                    return;
                } else {
                    btnCancel.setEnabled(true);
                }
                if (stateOnline) {
                    connectMenuItem.setEnabled(false);
                    disconnectMenuItem.setEnabled(true);
                    scanMenuItem.setEnabled(true);
                    txtSearch.setEnabled(true);
                    btnSearch.setEnabled(true);
                    btnDownload.setEnabled(true);
                } else {
                    connectMenuItem.setEnabled(true);
                    disconnectMenuItem.setEnabled(false);
                    scanMenuItem.setEnabled(false);
                    txtSearch.setEnabled(false);
                    btnSearch.setEnabled(false);
                    btnDownload.setEnabled(false);
                }
                if (stateScan) {
                    scanMenuItem.setEnabled(false);
                }
            }
        });
    }

    /**
     * ログとステータスラベルにメッセージを書き出す
     */
    public void putMessage(String msg) {
        statusLabel.setText(msg);
        taOutput.append(msg + "\n");
        taOutput.setCaretPosition(taOutput.getText().length());
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        mainPanel = new javax.swing.JPanel();
        jSplitPane = new javax.swing.JSplitPane();
        tabPane1 = new javax.swing.JTabbedPane();
        panelSearch = new javax.swing.JPanel();
        txtSearch = new javax.swing.JTextField();
        btnSearch = new javax.swing.JButton();
        btnDownload = new javax.swing.JButton();
        scrollSearch = new javax.swing.JScrollPane();
        tableSearch = new javax.swing.JTable();
        tabPane2 = new javax.swing.JTabbedPane();
        panelTasks = new javax.swing.JPanel();
        btnCancel = new javax.swing.JButton();
        scrollTasks = new javax.swing.JScrollPane();
        tableTasks = new javax.swing.JTable();
        panelLog = new javax.swing.JPanel();
        scrollLog = new javax.swing.JScrollPane();
        taOutput = new javax.swing.JTextArea();
        btnClear = new javax.swing.JButton();
        statusPanel = new javax.swing.JPanel();
        statusLabel = new javax.swing.JLabel();
        MenuBar = new javax.swing.JMenuBar();
        fileMenu = new javax.swing.JMenu();
        scanMenuItem = new javax.swing.JMenuItem();
        quitMenuItem = new javax.swing.JMenuItem();
        networkMenu = new javax.swing.JMenu();
        connectMenuItem = new javax.swing.JMenuItem();
        disconnectMenuItem = new javax.swing.JMenuItem();
        optionMenuItem = new javax.swing.JMenuItem();
        helpMenu = new javax.swing.JMenu();
        aboutMenuItem = new javax.swing.JMenuItem();

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle("P4UClient");
        setName("P4Uframe"); // NOI18N
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });
        getContentPane().setLayout(new java.awt.GridBagLayout());

        mainPanel.setLayout(new java.awt.GridBagLayout());

        jSplitPane.setDividerLocation(300);
        jSplitPane.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);

        tabPane1.setFont(tabPane1.getFont().deriveFont(tabPane1.getFont().getSize()+1f));
        tabPane1.setMinimumSize(new java.awt.Dimension(100, 100));
        tabPane1.setPreferredSize(new java.awt.Dimension(200, 150));

        panelSearch.setLayout(new java.awt.GridBagLayout());

        txtSearch.setFont(txtSearch.getFont().deriveFont(txtSearch.getFont().getSize()+2f));
        txtSearch.setMargin(new java.awt.Insets(3, 3, 3, 3));
        txtSearch.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                searchAction(evt);
            }
        });
        txtSearch.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                slectTextAction(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        panelSearch.add(txtSearch, gridBagConstraints);

        btnSearch.setMnemonic('S');
        btnSearch.setText("検索(S)");
        btnSearch.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                searchAction(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        panelSearch.add(btnSearch, gridBagConstraints);

        btnDownload.setMnemonic('D');
        btnDownload.setText("取得(D)");
        btnDownload.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                downloadAction(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        panelSearch.add(btnDownload, gridBagConstraints);

        tableSearch.setFont(tableSearch.getFont().deriveFont(tableSearch.getFont().getSize()+1f));
        tableSearch.setModel(modelSearch);
        tableSearch.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        scrollSearch.setViewportView(tableSearch);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 3;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 0.5;
        panelSearch.add(scrollSearch, gridBagConstraints);

        tabPane1.addTab(" 検索 (A)", panelSearch);

        tabPane1.setMnemonicAt(0, 'A');

        jSplitPane.setLeftComponent(tabPane1);

        panelTasks.setLayout(new java.awt.GridBagLayout());

        btnCancel.setMnemonic('N');
        btnCancel.setText("中断(N)");
        btnCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelAction(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        panelTasks.add(btnCancel, gridBagConstraints);

        tableTasks.setFont(tableTasks.getFont().deriveFont(tableTasks.getFont().getSize()+1f));
        tableTasks.setModel(modelTasks);
        tableSearch.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        scrollTasks.setViewportView(tableTasks);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 0.5;
        panelTasks.add(scrollTasks, gridBagConstraints);

        tabPane2.addTab(" タスク (T)", panelTasks);

        panelLog.setLayout(new java.awt.GridBagLayout());

        taOutput.setColumns(20);
        taOutput.setEditable(false);
        taOutput.setFont(taOutput.getFont());
        taOutput.setLineWrap(true);
        taOutput.setRows(5);
        scrollLog.setViewportView(taOutput);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 0.5;
        panelLog.add(scrollLog, gridBagConstraints);

        btnClear.setMnemonic('C');
        btnClear.setText("クリア(C)");
        btnClear.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                clearAction(evt);
            }
        });
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        panelLog.add(btnClear, gridBagConstraints);

        tabPane2.addTab(" ログ (L)", panelLog);

        tabPane2.setMnemonicAt(0, 'T');
        tabPane2.setMnemonicAt(1, 'L');

        jSplitPane.setRightComponent(tabPane2);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 0.5;
        mainPanel.add(jSplitPane, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 0.5;
        getContentPane().add(mainPanel, gridBagConstraints);

        statusPanel.setLayout(new java.awt.GridBagLayout());

        statusLabel.setText("jLabel1");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.EAST;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 0.5;
        gridBagConstraints.insets = new java.awt.Insets(5, 5, 5, 5);
        statusPanel.add(statusLabel, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridheight = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 0.5;
        getContentPane().add(statusPanel, gridBagConstraints);

        fileMenu.setMnemonic('F');
        fileMenu.setText("ファイル(F)");

        scanMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        scanMenuItem.setMnemonic('S');
        scanMenuItem.setText("スキャン(S)");
        scanMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mainActions(evt);
            }
        });
        fileMenu.add(scanMenuItem);

        quitMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_Q, java.awt.event.InputEvent.CTRL_MASK));
        quitMenuItem.setMnemonic('X');
        quitMenuItem.setText("終了(X)");
        quitMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mainActions(evt);
            }
        });
        fileMenu.add(quitMenuItem);

        MenuBar.add(fileMenu);

        networkMenu.setMnemonic('N');
        networkMenu.setText("ネットワーク(N)");

        connectMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_C, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        connectMenuItem.setMnemonic('C');
        connectMenuItem.setText("接続(C)");
        connectMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mainActions(evt);
            }
        });
        networkMenu.add(connectMenuItem);

        disconnectMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_D, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        disconnectMenuItem.setMnemonic('D');
        disconnectMenuItem.setText("切断(D)");
        disconnectMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mainActions(evt);
            }
        });
        networkMenu.add(disconnectMenuItem);

        optionMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_O, java.awt.event.InputEvent.CTRL_MASK));
        optionMenuItem.setMnemonic('O');
        optionMenuItem.setText("設定(O)");
        optionMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mainActions(evt);
            }
        });
        networkMenu.add(optionMenuItem);

        MenuBar.add(networkMenu);

        helpMenu.setMnemonic('H');
        helpMenu.setText("ヘルプ(H)");

        aboutMenuItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_A, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        aboutMenuItem.setMnemonic('A');
        aboutMenuItem.setText("P4Uについて(A)");
        aboutMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mainActions(evt);
            }
        });
        helpMenu.add(aboutMenuItem);

        MenuBar.add(helpMenu);

        setJMenuBar(MenuBar);

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-800)/2, (screenSize.height-600)/2, 800, 600);
    }// </editor-fold>//GEN-END:initComponents
    /**
     * 基本的なアクション
     */
    private void mainActions(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mainActions
        final String action = evt.getActionCommand();
        SwingWorker worker = new SwingWorker() {

            @Override
            protected Object doInBackground() {
                if (action.equals("接続(C)")) {
                    MainApp.online();
                } else if (action.equals("スキャン(S)")) {
                    MainApp.update();
                } else if (action.equals("切断(D)")) {
                    MainApp.offline();
                } else if (action.equals("終了(X)")) {
                    // formWindowClosing みたいなの作って呼びたい
                } else if (action.equals("設定(O)")) {
                    OptionBox.showOptionBox();
                } else if (action.equals("P4Uについて(A)")) {
                    AboutBox.showAboutBox();
                }
                
                return null;
            }
        };
        worker.execute();
}//GEN-LAST:event_mainActions

    /**
     * テキストボックスを選択
     */
    private void slectTextAction(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_slectTextAction
        javax.swing.JTextField textBox = (javax.swing.JTextField) evt.getSource();
        textBox.selectAll();
}//GEN-LAST:event_slectTextAction

    /**
     * ファイルを検索する
     */
    private void searchAction(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_searchAction
        final String word = txtSearch.getText();
        SwingWorker worker = new SwingWorker() {

            @Override
            protected Object doInBackground() {
                MainApp.search(word);
                return null;
            }
        };
        worker.execute();
}//GEN-LAST:event_searchAction

    /**
     * 選択したファイルをダウンロード
     */
    private void downloadAction(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_downloadAction
        // 選択チェック
        int select = tableSearch.getSelectedRow();
        if (select == -1) {
            return;
        }
        final FileItem item = modelSearch.getItem(select);
        // 実行
        SwingWorker worker = new SwingWorker() {

            @Override
            protected Object doInBackground() {
                MainApp.download(item);
                return null;
            }
        };
        worker.execute();
}//GEN-LAST:event_downloadAction

    /**
     * ログをクリア
     */
    private void clearAction(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_clearAction
        taOutput.setText("");
}//GEN-LAST:event_clearAction

    /**
     * タスクを中断
     */
    private void cancelAction(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelAction
        // 選択チェック
        int select = tableTasks.getSelectedRow();
        if (select == -1) {
            return;
        }
        final TaskAbstract item = modelTasks.getItem(select);
        // 実行
        SwingWorker worker = new SwingWorker() {

            @Override
            protected Object doInBackground() {
                MainApp.cancel(item);
                return null;
            }
        };
        worker.execute();
}//GEN-LAST:event_cancelAction

    /**
     * ウィンドウを閉じる
     */
    private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
        // 終了チェック
        int option = JOptionPane.showConfirmDialog(
                this, "本当に終了しちゃいますよ？",
                "P4UClient - 確認", JOptionPane.YES_NO_OPTION);
        if (option != JOptionPane.YES_OPTION) {
            return;
        }
        // 実行
        SwingWorker worker = new SwingWorker() {

            @Override
            protected Object doInBackground() {
                MainApp.exit();
                return null;
            }
        };
        worker.execute();
    }//GEN-LAST:event_formWindowClosing

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenuBar MenuBar;
    private javax.swing.JMenuItem aboutMenuItem;
    private javax.swing.JButton btnCancel;
    private javax.swing.JButton btnClear;
    private javax.swing.JButton btnDownload;
    private javax.swing.JButton btnSearch;
    private javax.swing.JMenuItem connectMenuItem;
    private javax.swing.JMenuItem disconnectMenuItem;
    private javax.swing.JMenu fileMenu;
    private javax.swing.JMenu helpMenu;
    private javax.swing.JSplitPane jSplitPane;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JMenu networkMenu;
    private javax.swing.JMenuItem optionMenuItem;
    private javax.swing.JPanel panelLog;
    private javax.swing.JPanel panelSearch;
    private javax.swing.JPanel panelTasks;
    private javax.swing.JMenuItem quitMenuItem;
    private javax.swing.JMenuItem scanMenuItem;
    private javax.swing.JScrollPane scrollLog;
    private javax.swing.JScrollPane scrollSearch;
    private javax.swing.JScrollPane scrollTasks;
    private javax.swing.JLabel statusLabel;
    private javax.swing.JPanel statusPanel;
    private javax.swing.JTextArea taOutput;
    private javax.swing.JTabbedPane tabPane1;
    private javax.swing.JTabbedPane tabPane2;
    private javax.swing.JTable tableSearch;
    private javax.swing.JTable tableTasks;
    private javax.swing.JTextField txtSearch;
    // End of variables declaration//GEN-END:variables
}
